<?php
/*
Plugin Name: Cloudinary
Plugin URI: http://cloudinary.com/
Description: Cloudinary allows you to upload your images to the cloud. They'll be available to your visitors through a fast content delivery network, improving your website's loading speed and overall user experience. With Cloudinary, you can transform uploaded images without leaving Wordpress - apply effects (sharpen, gray scale, sepia, and more), smart cropping and re-sizing (including face detection based cropping), and much more.

Version:  1.0.5
Author: Cloudinary Ltd.
Author URI: http://cloudinary.com/
*/
define('cloudinary_VERSION', '1.0.5');
define('cloudinary_PLUGIN_URL', plugin_dir_url( __FILE__ ));
require "cloudinary_api.php" ;
require "uploader.php" ;

define ('CLOUDINARY_BASE_URL', "https://cloudinary.com");
define ('CLOUDINARY_UNIQUE_ID', "cloudinary-image-management-and-manipulation-in-the-cloud-cdn");


function cloudinary_include_assets() {
  $cloudinary_js_dir = plugins_url('/js', __FILE__); 
  wp_enqueue_script('jquery');
  wp_enqueue_script('cld-form', $cloudinary_js_dir . '/jquery.form.js');

  $cloudinary_css_dir = plugins_url('/css', __FILE__); 
  wp_enqueue_style('cld-style', $cloudinary_css_dir . '/cloudinary.css?cv=' . cloudinary_VERSION);
}


class CloudinaryPlugin
{
  function CloudinaryPlugin() {
    $this->cloudinary_upgrade();
    register_uninstall_hook('uninstall.php', '');
    
    add_action('admin_init', array($this, 'config'));    
    add_action('admin_menu', array($this, 'cloudinary_options_page'));
    
    add_filter('manage_media_columns', array($this, 'media_lib_add_upload_column') );
    add_action('manage_media_custom_column', array($this, 'media_lib_upload_column_value'), 0, 2);
    add_action('admin_footer-upload.php', array($this, 'media_lib_upload_admin_footer'));    
    add_action('load-upload.php', array($this, 'media_lib_upload_action'));
    add_action('admin_notices', array($this, 'media_lib_upload_notices'));
    
    add_action('media_buttons', array($this, 'media_cloudinary'), 11);
    add_action('wp_ajax_cloudinary_update_options', array($this, 'update_options'));
  }

  /* Upgrade */
  function cloudinary_upgrade() {
    $current_version = get_option('cloudinary_version');
    if ($current_version && version_compare($current_version, '1.0.4', '<')) {
      /* cloudinary_url used to be stored in a configuration table. migrate to an option */
      global $wpdb;
      $table_name_account = $wpdb->prefix . "cloudinary_config";
      $myrows = $wpdb->get_results( "SELECT * FROM $table_name_account where user_id = 1" );
      if(!empty($myrows)){
        $cloudinary_url = trim($myrows[0]->cloudinary_url);
        update_option('cloudinary_url', $cloudinary_url);
      }                
      /* drop table in database */
      $wpdb->query("DROP TABLE IF EXISTS $table_name_account");
    }
    update_option('cloudinary_version', cloudinary_VERSION);
  }

  /* Configure Cloudinary integration if settings available */ 
  function config() {
    $cloudinary_url = get_option('cloudinary_url');
    if($cloudinary_url){
      Cloudinary::config_from_url($cloudinary_url);
    }    
  }    
  
  /* Configure menus */
  function cloudinary_options_page() {

    $this->config();
    $configured = Cloudinary::config_get("api_secret") && Cloudinary::config_get("cloud_name") && Cloudinary::config_get("api_key");
    $settings_item = CLOUDINARY_UNIQUE_ID . "/options.php";
    $library_item = CLOUDINARY_UNIQUE_ID . "/library.php";
    $main_action = $configured ? $library_item : $settings_item;
    add_menu_page('Cloudinary Menu', 'Cloudinary', 'manage_options', $main_action, NULL, plugins_url(CLOUDINARY_UNIQUE_ID . '/images/favicon.png'));
    if ($configured){
      add_submenu_page($main_action, "Cloudinary Media Library", "Media library", 'publish_pages', $main_action);
      add_submenu_page($main_action, "Cloudinary Settings", "Settings", 'manage_options', $settings_item);
    } else {
      add_submenu_page($main_action, "Cloudinary Settings", "Settings", 'manage_options', $main_action);
      add_submenu_page($main_action, "Cloudinary Media Library", "Media library", 'publish_pages', $library_item);
    }
  }

  function update_options() {
    if ( empty($_POST) || !wp_verify_nonce($_POST['_wpnonce'],'cloudinary_update_options') ) {
       echo 'Sorry, your nonce did not verify.';
    } else {    
      $cloudinary_url = str_replace("CLOUDINARY_URL=", "", trim($_POST['cloudinary_url']));
      Cloudinary::config_from_url($cloudinary_url);
      $configured = Cloudinary::config_get("api_secret") && Cloudinary::config_get("cloud_name") && Cloudinary::config_get("api_key");
      if ($configured) {
        update_option('cloudinary_url', $cloudinary_url);
        $url = $this->prepare_cloudinary_media_lib_url("check");
        $response = wp_remote_get($url);
        if ($response["response"]["code"] == "200") {
          echo 'success';
        } else {
          echo 'Cannot access Cloudinary (error ' . $response["response"]["code"] . ") - Verify your CLOUDINARY_URL";
        }        
      } else {
        echo 'Invalid CLOUDINARY_URL. Must match the following format: cloudinary://api_key:api_secret@cloud_name';
      }
    }
    die();    
  }
  
  function init_media_lib_integration($xdmremote, $autoShow) {
    $cloudinary_js_dir = plugins_url('/js', __FILE__); 
    wp_enqueue_script('jquery');
    wp_enqueue_script('cld-xdm', $cloudinary_js_dir . '/easyXDM.min.js');
    wp_enqueue_script('cld-json2', $cloudinary_js_dir . '/json2.min.js');
    wp_enqueue_script('cld-js', $cloudinary_js_dir . '/cloudinary.js?cv=' . cloudinary_VERSION);
    $cloudinary_css_dir = plugins_url('/css', __FILE__); 
    wp_enqueue_style('cld-css', $cloudinary_css_dir . '/cloudinary.css?cv=' . cloudinary_VERSION);
	

    $xdmbase = plugins_url('', __FILE__);
    $xdmremotehelper = CLOUDINARY_BASE_URL . "/easyXDM.name.html";
    $xdmautoshow = $autoShow ? "true" : "false";
    return "<link href='" . $cloudinary_css_dir . "/cloudinary.css?cv=" . cloudinary_VERSION . "' media='screen' rel='stylesheet' type='text/css' />" .
     "<span id='cloudinary-library-config' data-base='$xdmbase' data-remote='$xdmremote' data-remotehelper='$xdmremotehelper' data-autoshow=$xdmautoshow></span>";
  }
  
  function media_cloudinary($editor_id = 'content') {
    $context = apply_filters('media_buttons_context', __('Cloudinary Upload/Insert'));
    $xdmremote = $this->prepare_cloudinary_media_lib_url("wp_post");

    echo $this->init_media_lib_integration($xdmremote, false) .
         '<a href="#" class="cloudinary_add_media" id="' . esc_attr( $editor_id ) . '-add_media" ' .
         'title="' . esc_attr__( 'Add Media from Cloudinary' ) . '">' . $context . '</a>';
  }
  
  function media_lib_add_upload_column( $cols ) {
    $cols["media_url"] = "Cloudinary";
    return $cols;
  }

  function media_lib_upload_column_value( $column_name) {
    if ( $column_name == "media_url") {
      $cloudinary_js_dir = plugins_url('/js', __FILE__); 
      wp_enqueue_script('jquery');
      wp_enqueue_script('cld-media', $cloudinary_js_dir . '/media.js');

      $action_url = wp_nonce_url("?", "bulk-media"); 
      echo "<a href='$action_url'>Upload to Cloudinary</a>";      
    }
  }
    
  function media_lib_upload_admin_footer() {
    global $post_type;
    $img_l = get_bloginfo('url') . "/wp-content/plugins/" . CLOUDINARY_UNIQUE_ID . "/images/ajax-loader.gif";
    ?>
    <script type="text/javascript">
    jQuery(document).ready(function() {
      jQuery('<option>').val('cloudinary').text('<?php _e('Upload to Cloudinary')?>').appendTo("select[name='action']");
      jQuery('body').prepend('<div class="black_overlay" id="fade" style="display: none;"></div><div style="background-color: white; display: none;" id="loading-image"><table border="0" cellspacing="0" cellpadding="5"><tbody><tr><td><img alt="Loading..." src="<?php echo $img_l; ?>"></td><td>Loading..Please Wait!</td></tr></tbody></table></div>');
    });
    </script>
    <?php
  }

  function media_lib_upload_action() {
    $wp_list_table = _get_list_table('WP_Media_List_Table');
    $pagenum = $wp_list_table->get_pagenum();
    $action = $wp_list_table->current_action();
    global $post_type, $pagenow;
    if($pagenow == 'upload.php' && isset($_REQUEST['cloud_upload']) && (int) $_REQUEST['cloud_upload']) {
      check_admin_referer('bulk-media');
      // Single image upload
      global $wpdb;
      $att_id = $_REQUEST['cloud_upload'];
      $this->config();
      $url1 = wp_get_attachment_metadata($att_id);
      $img_path = $url1['file'];
      $img_ex = explode('/',$img_path);
      $img_name = explode('.',$img_ex['2']);
      
      $upload_dir = wp_upload_dir();
      $full_path = $upload_dir['basedir'].'/'.$img_path;
      $validate_img =  @getimagesize($full_path);
      if(!empty($validate_img) && is_array($validate_img)){
        if (!Cloudinary::config_get("api_secret")) {
               echo "Please setup environment to upload images to cloudinary";
               return;
        }
        try {
          $result = CloudinaryUploader::upload($full_path,array('use_filename'=>True)); 
        } catch(Exception $e) {
          echo $e->getMessage();
          exit();
        }
        $_REQUEST = array();
        $location = add_query_arg( array('cloudinary' => 1), $sendback );
        wp_redirect( $location );
        exit();
      }
    }
    if($action === 'cloudinary') {
      // Multiple images upload
      check_admin_referer('bulk-media');
      if ( isset($_REQUEST['media'] ) ) {
        $post_ids = $_REQUEST['media'];
      } elseif ( isset( $_REQUEST['ids'] ) ) {
        $post_ids = explode( ',', $_REQUEST['ids'] );
      }
      global $current_user;  
      get_currentuserinfo();
      $exported = 0;
      global $wpdb;
      $this->config();
      foreach( $post_ids as $k =>  $post_id ) {
        $url = wp_get_attachment_metadata( $post_id );
        $img_ex = explode('/',$url['file']);
        $img_name = explode('.',$img_ex['2']);
        $img_path = $url['file'];
        $upload_dir = wp_upload_dir();
        $full_path = $upload_dir['basedir'].'/'.$img_path;
        $validate_img =  @getimagesize($full_path);
        if(!empty($validate_img) && is_array($validate_img)){
          if (!Cloudinary::config_get("api_secret")) {
             echo "Please setup environment for upload image to cloudinary";
             return;
          }
          try{
            $result = CloudinaryUploader::upload($full_path,array('use_filename'=>True));
          }
          catch(Exception $e){
            echo $e->getMessage();
            exit();
          }
          $exported++;
        }
      }
      $location = add_query_arg( array('cloudinary' => $exported), $sendback );  
      wp_redirect( $location );
      exit();
    }
  }

  function media_lib_upload_notices() {
    global $post_type, $pagenow;
    if ($pagenow == 'upload.php' && $post_type == 'attachment' && 
      isset($_REQUEST['cloudinary']) && (int) $_REQUEST['cloudinary']) {
      $image = 'Images'; 
      if(((int) $_REQUEST['cloudinary']) == 1) {$image = 'Image';}
      $message = sprintf( '%s '.$image.' uploaded to cloudinary.', number_format_i18n( $_REQUEST['cloudinary'] ) );
      echo "<div class=\"updated\"><p>{$message}</p></div>";
    }
    if ($pagenow == 'upload.php' && isset($_REQUEST['cloud_upload']) && (int) $_REQUEST['cloud_upload']) {
      $message = "Sorry, this file format is not supported.";
      echo "<div class=\"updated\"><p>{$message}</p></div>";
    }
  }
 
  function prepare_cloudinary_media_lib_url($mode) {
    $this->config();
    $params = array("timestamp" => time(), "mode"=>$mode);
    $params["signature"] = Cloudinary::api_sign_request($params, Cloudinary::config_get("api_secret"));
    $params["api_key"] = Cloudinary::config_get("api_key");
    $query = http_build_query($params);
    return CLOUDINARY_BASE_URL . "/console/media_library/cms?$query";
  }   
}

$cloudinary_plugin = new CloudinaryPlugin();

